MOVING_RAID_MEMBER = nil;
TARGET_RAID_SLOT = nil;
CT_RATab_AutoPromotions = { };
CT_RATab_RaidUITooltip = "The ability to drag out classes and groups from the Blizzard Raid UI has been disabled in favor for CT_RaidAssist's Player Targets for added efficiency. You can set up to 10 personal players as Player Targets by going to the CTRaid tab and right-clicking the player in question.";

function CT_RATabFrame_OnHide()
	-- If there's a selected button then call the onmouseup function on it when the frame is hidden
	if ( MOVING_RAID_MEMBER ) then
		CT_RATabButton_OnDragStop(MOVING_RAID_MEMBER);
	end
end

function CT_RATabFrame_OnEvent()
	RaidFrame_OnEvent();
	if ( event == "UNIT_LEVEL" ) then
		local id, found = gsub(arg1, "raid([0-9]+)", "%1");
		if ( found == 1 ) then
			CT_RATabFrame_UpdateLevel(id);
		end
	end
	if ( event == "UNIT_HEALTH" ) then
		local id, found = gsub(arg1, "raid([0-9]+)", "%1");
		if ( found == 1 ) then
			CT_RATabFrame_UpdateHealth(id);
		end
	end
end

function CT_RATabFrame_Update()
	-- Update raid group labels
	if ( GetNumRaidMembers() == 0 ) then
		for i=1, NUM_RAID_GROUPS do
			getglobal("CT_RATabGroup"..i):Hide();
		end
		CT_RATabAddMemberButton:Hide();
	else
		for i=1, NUM_RAID_GROUPS do
			getglobal("CT_RATabGroup"..i):Show();
		end
		CT_RATabAddMemberButton:Show();
		if ( IsRaidLeader() or IsRaidOfficer() ) then
			CT_RATabAddMemberButton:Enable();
		else
			CT_RATabAddMemberButton:Disable();
		end
	end

	-- Reset group index counters;
	for i=1, NUM_RAID_GROUPS do
		getglobal("CT_RATabGroup"..i).nextIndex = 1;
	end
	-- Clear out all the slots buttons
	CT_RATab_ResetSlotButtons();

	-- Fill out buttons
	local numRaidMembers = GetNumRaidMembers();
	local raidGroup, color;
	local buttonName, buttonLevel, buttonClass, buttonClassText, buttonRank;
	local name, rank, subgroup, level, class, fileName, zone, online, isDead;
	for i=1, MAX_RAID_MEMBERS do
		button = getglobal("CT_RATabButton"..i);
		if ( i <= numRaidMembers ) then
			name, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(i);
			raidGroup = getglobal("CT_RATabGroup"..subgroup);
			-- To prevent errors when the server hiccups
			if ( raidGroup.nextIndex <= MEMBERS_PER_RAID_GROUP ) then
				buttonName = getglobal("CT_RATabButton"..i.."Name");
				buttonClass = getglobal("CT_RATabButton"..i.."Class");
				buttonClassText = getglobal("CT_RATabButton"..i.."ClassText");
				buttonLevel = getglobal("CT_RATabButton"..i.."Level");
				buttonRank = getglobal("CT_RATabButton"..i.."Rank");
				button.id = i;
				
				button.name = name;
				button.unit = "raid"..i;
				button.class = fileName;
				
				if ( level == 0 ) then
					level = "";
				end
				
				if ( not name ) then
					name = UNKNOWN;
				end
				buttonName:SetText(name);
				buttonClass:SetText(class or "");
				buttonLevel:SetText(level or "");
				
				if ( online ) then
					if ( UnitIsGhost("raid" .. i) or UnitIsDead("raid" .. i) ) then
						buttonName:SetVertexColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
						buttonClassText:SetVertexColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
						buttonLevel:SetVertexColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
					else
						color = RAID_CLASS_COLORS[fileName];
						if ( color ) then
							buttonName:SetVertexColor(color.r, color.g, color.b);
							buttonClassText:SetVertexColor(color.r, color.g, color.b);
							buttonLevel:SetVertexColor(color.r, color.g, color.b);
						end
					end
				else
					buttonName:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
					buttonClassText:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
					buttonLevel:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
				end

				if ( rank == 2 ) then
					buttonRank:SetText(RAID_LEADER_TOKEN);
				elseif ( rank == 1 ) then
					buttonRank:SetText(RAID_ASSISTANT_TOKEN);
				else 
					buttonRank:SetText("");
				end
			
				-- Anchor button to slot
				if ( MOVING_RAID_MEMBER ~= button  ) then
					button:SetPoint("TOPLEFT", "CT_RATabGroup"..subgroup.."Slot"..raidGroup.nextIndex, "TOPLEFT", 0, 0);
				end
				
				-- Save slot for future use
				button.slot = "CT_RATabGroup"..subgroup.."Slot"..raidGroup.nextIndex;
				-- Save the button's subgroup too
				button.subgroup = subgroup;
				-- Tell the slot what button is in it
				getglobal("CT_RATabGroup"..subgroup.."Slot"..raidGroup.nextIndex).button = button:GetName();
				raidGroup.nextIndex = raidGroup.nextIndex + 1;
				button:SetID(i);
				button:Show();
			end
		else
			button:Hide();
		end
	end
end

function CT_RATabFrame_UpdateLevel(id)
	local unit = "raid"..id;
	local buttonLevel = getglobal("CT_RATabButton"..id.."Level");
	buttonLevel:SetText(UnitLevel(unit));
end

function CT_RATabFrame_UpdateHealth(id)
	local name, rank, subgroup, level, class, fileName, zone, online, isDead = GetRaidRosterInfo(id);

	local buttonName = getglobal("CT_RATabButton"..id.."Name");
	local buttonClassText = getglobal("CT_RATabButton"..id.."ClassText");
	local buttonLevel = getglobal("CT_RATabButton"..id.."Level");

	if ( online ) then
		if ( UnitIsGhost("raid" .. id) or UnitIsDead("raid" .. id) ) then
			buttonName:SetVertexColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
			buttonClassText:SetVertexColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
			buttonLevel:SetVertexColor(RED_FONT_COLOR.r, RED_FONT_COLOR.g, RED_FONT_COLOR.b);
		else
			color = RAID_CLASS_COLORS[fileName];
			if ( color ) then
				buttonName:SetVertexColor(color.r, color.g, color.b);
				buttonClassText:SetVertexColor(color.r, color.g, color.b);
				buttonLevel:SetVertexColor(color.r, color.g, color.b);
			end
		end
	else
		buttonName:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
		buttonClassText:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
		buttonLevel:SetVertexColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
	end
end

function CT_RATabFrame_OnUpdate(elapsed)
	if ( MOVING_RAID_MEMBER ) then
		local button, slot;
		TARGET_RAID_SLOT = nil;
		for i=1, NUM_RAID_GROUPS do
			for j=1, MEMBERS_PER_RAID_GROUP do
				slot = getglobal("CT_RATabGroup"..i.."Slot"..j);
				if ( MouseIsOver(slot) ) then
					slot:LockHighlight();
					TARGET_RAID_SLOT = slot;
				else
					slot:UnlockHighlight();
				end
			end
		end
	end
end

function CT_RATabButton_OnDragStart()
	if ( not IsRaidLeader() ) then
		return;
	end
	local cursorX, cursorY = GetCursorPosition();
	this:ClearAllPoints();
	this:SetPoint("CENTER", "UIParent", "BOTTOMLEFT", cursorX*GetScreenWidthScale(), cursorY*GetScreenHeightScale());
	this:StartMoving();
	MOVING_RAID_MEMBER = this;
	SetRaidRosterSelection(this.id);
end

function CT_RATabButton_OnDragStop(raidButton)
	if ( not IsRaidLeader() ) then
		return;
	end
	if ( not raidButton ) then
		raidButton = this;
	end
	raidButton:StopMovingOrSizing();
	MOVING_RAID_MEMBER = nil;
	if ( TARGET_RAID_SLOT and TARGET_RAID_SLOT:GetParent():GetID() ~= raidButton.subgroup ) then
		if (TARGET_RAID_SLOT.button) then
			local button = getglobal(TARGET_RAID_SLOT.button);
			--button:SetPoint("TOPLEFT", this, "TOPLEFT", 0, 0);
			SwapRaidSubgroup(raidButton:GetID(), button:GetID());
		else
			local slot = TARGET_RAID_SLOT:GetParent():GetName().."Slot"..TARGET_RAID_SLOT:GetParent().nextIndex;
			raidButton:SetPoint("TOPLEFT", slot, "TOPLEFT", 0, 0);
			TARGET_RAID_SLOT:UnlockHighlight();
			SetRaidSubgroup(raidButton:GetID(), TARGET_RAID_SLOT:GetParent():GetID());
		end
	else
		if ( TARGET_RAID_SLOT ) then
			TARGET_RAID_SLOT:UnlockHighlight();
		end
		raidButton:SetPoint("TOPLEFT", raidButton.slot, "TOPLEFT", 0, 0);
	end
end

function CT_RATabButton_OnClick(button)
	if ( button == "LeftButton" ) then
		local unit = "raid"..this.id;
		if ( SpellIsTargeting() ) then
			SpellTargetUnit(unit);
		elseif ( CursorHasItem() ) then
			DropItemOnUnit(unit);
		else
			TargetUnit(unit);
		end
	else
		HideDropDownMenu(1);
		if ( this.id and this.name ) then
			FriendsDropDown.initialize = CT_RATabDropDown_Initialize;
			FriendsDropDown.displayMode = "MENU";
			ToggleDropDownMenu(1, nil, FriendsDropDown, "cursor");
		end
	end
end

function CT_RATabButton_OnEnter()
	if ( SpellIsTargeting() ) then
		if ( SpellCanTargetUnit(this.unit) ) then
			SetCursor("CAST_CURSOR");
		else
			SetCursor("CAST_ERROR_CURSOR");
		end
	end

	GameTooltip_SetDefaultAnchor(GameTooltip, this);
	if ( GameTooltip:SetUnit(this.unit) ) then
		this.updateTooltip = TOOLTIP_UPDATE_TIME;
	else
		this.updateTooltip = nil;
	end

	this.r, this.g, this.b = GameTooltip_UnitColor(this.unit);
	GameTooltipTextLeft1:SetTextColor(this.r, this.g, this.b);
end

function CT_RATabDropDown_Initialize()
	UnitPopup_ShowMenu(getglobal(UIDROPDOWNMENU_OPEN_MENU), "RAID", this.unit, this.name, this.id);
	local info = {};
	info.text = "Auto-Promote";
	info.tooltipTitle = "Auto-Promote";
	info.tooltipText = "When checked, this player is automatically promoted when he or she joins the raid.";
	info.checked = CT_RATab_AutoPromotions[this.name];
	info.value = this.id;
	info.func = CT_RATab_AutoPromote_OnClick;
	UIDropDownMenu_AddButton(info);
end

function CT_RATab_AutoPromote_OnClick()
	local name, rank = GetRaidRosterInfo(this.value);
	CT_RATab_AutoPromotions[name] = not CT_RATab_AutoPromotions[name];
	if ( CT_RA_Level and CT_RA_Level >= 2 and CT_RATab_AutoPromotions[name] and rank < 1 ) then
		PromoteToAssistant(name);
		CT_RA_Print("<CTRaid> Auto-Promoted |c00FFFFFF" .. name .. "|r.", 1, 0.5, 0);
	end
end

function CT_RATab_ResetSlotButtons()
	for i=1, NUM_RAID_GROUPS do
		for j=1, MEMBERS_PER_RAID_GROUP do
			getglobal("CT_RATabGroup"..i.."Slot"..j).button = nil;
		end
	end
end

RaidGroupFrame_Update = CT_RATabFrame_Update;

function CT_RATab_LoadDropDown()
	UIDropDownMenu_Initialize(this, CT_RATab_InitializeDropDown);
	UIDropDownMenu_SetWidth(135);
end

function CT_RATab_InitializeDropDown()
	local info = { };
	info.text = LOOT_FREE_FOR_ALL;
	info.value = "freeforall";
	info.tooltipTitle = LOOT_FREE_FOR_ALL;
	info.tooltipText = "Set the default loot type to Free for All, which will make CT_RaidAssist automatically set your loot type to the selected option upon creating a raid.";
	info.func = CT_RATab_ClickDropDown;
	UIDropDownMenu_AddButton(info);
	
	info = { };
	info.text = LOOT_ROUND_ROBIN;
	info.value = "roundrobin";
	info.tooltipTitle = LOOT_ROUND_ROBIN;
	info.tooltipText = "Set the default loot type to Round Robin, which will make CT_RaidAssist automatically set your loot type to the selected option upon creating a raid.";
	info.func = CT_RATab_ClickDropDown;
	UIDropDownMenu_AddButton(info);
	
	info = { };
	info.text = LOOT_MASTER_LOOTER;
	info.value = "master";
	info.tooltipTitle = LOOT_MASTER_LOOTER;
	info.tooltipText = "Set the default loot type to Master Looter, which will make CT_RaidAssist automatically set your loot type to the selected option upon creating a raid.";
	info.func = CT_RATab_ClickDropDown;
	UIDropDownMenu_AddButton(info);

	info = { };
	info.text = LOOT_GROUP_LOOT;
	info.value = "group";
	info.tooltipTitle = LOOT_GROUP_LOOT;
	info.tooltipText = "Set the default loot type to Group Loot, which will make CT_RaidAssist automatically set your loot type to the selected option upon creating a raid.";
	info.func = CT_RATab_ClickDropDown;
	UIDropDownMenu_AddButton(info);
	
	info = { };
	info.text = LOOT_NEED_BEFORE_GREED;
	info.value = "needbeforegreed";
	info.tooltipTitle = LOOT_NEED_BEFORE_GREED;
	info.tooltipText = "Set the default loot type to Need Before Greed, which will make CT_RaidAssist automatically set your loot type to the selected option upon creating a raid.";
	info.func = CT_RATab_ClickDropDown;
	UIDropDownMenu_AddButton(info);
end

function CT_RATab_ClickDropDown()
	CT_RATab_SetDropDownID(this:GetID());
	CT_RATab_DefaultLootMethod = this:GetID();
	if ( CT_RA_Level >= 2 and GetNumRaidMembers() > 0 ) then
		if ( this.value == "master" ) then
			SetLootMethod("master", UnitName("player"));
		else
			SetLootMethod(this.value);
		end
	end
end

function CT_RATab_SetDropDownID(id)
	UIDropDownMenu_SetSelectedID(CT_RATabLootDropDown, id);
	if ( id == 1 ) then
		CT_RATabLootDropDownText:SetText(LOOT_FREE_FOR_ALL);
	elseif ( id == 2 ) then
		CT_RATabLootDropDownText:SetText(LOOT_ROUND_ROBIN);
	elseif ( id == 3 ) then
		CT_RATabLootDropDownText:SetText(LOOT_MASTER_LOOTER);
	elseif ( id == 4 ) then
		CT_RATabLootDropDownText:SetText(LOOT_GROUP_LOOT);
	elseif ( id == 5 ) then
		CT_RATabLootDropDownText:SetText(LOOT_NEED_BEFORE_GREED);
	end
end